@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -60,7 +60,9 @@ static bool l_InExceptionHandler = false;
 int Application::m_ArgC;
 char **Application::m_ArgV;
 double Application::m_StartTime;
+double Application::m_MainTime;
 bool Application::m_ScriptDebuggerEnabled = false;
+double Application::m_LastReloadFailed;
 
 /**
  * Constructor for the Application class.
@@ -329,8 +331,10 @@ void Application::OnShutdown(void)
 
 static void ReloadProcessCallbackInternal(const ProcessResult& pr)
 {
-	if (pr.ExitStatus != 0)
+	if (pr.ExitStatus != 0) {
+		Application::SetLastReloadFailed(Utility::GetTime());
 		Log(LogCritical, "Application", "Found error in config: reloading aborted");
+	}
 #ifdef _WIN32
 	else
 		Application::Exit(7); /* keep this exit code in sync with icinga-app */
@@ -671,7 +675,17 @@ void Application::SigAbrtHandler(int)
 		  << std::endl;
 
 	String fname = GetCrashReportFilename();
-	Utility::MkDir(Utility::DirName(fname), 0750);
+	String dirName = Utility::DirName(fname);
+
+	if (!Utility::PathExists(dirName)) {
+#ifndef _WIN32
+		if (mkdir(dirName.CStr(), 0700) < 0 && errno != EEXIST) {
+#else /*_ WIN32 */
+		if (mkdir(dirName.CStr()) < 0 && errno != EEXIST) {
+#endif /* _WIN32 */
+			std::cerr << "Could not create directory '" << dirName << "': Error " << errno << ", " << strerror(errno) << "\n";
+		}
+	}
 
 	bool interactive_debugger = Convert::ToBool(ScriptGlobal::Get("AttachDebugger"));
 
@@ -693,7 +707,7 @@ void Application::SigAbrtHandler(int)
 		ofs << "\n";
 		ofs.close();
 	} else {
-		Log(LogCritical, "Application", "Icinga 2 has terminated unexpeectedly. Attaching debugger...");
+		Log(LogCritical, "Application", "Icinga 2 has terminated unexpectedly. Attaching debugger...");
 	}
 
 	AttachDebugger(fname, interactive_debugger);
@@ -737,11 +751,21 @@ void Application::ExceptionHandler(void)
 #endif /* _WIN32 */
 
 	String fname = GetCrashReportFilename();
-	Utility::MkDir(Utility::DirName(fname), 0750);
+	String dirName = Utility::DirName(fname);
+
+	if (!Utility::PathExists(dirName)) {
+#ifndef _WIN32
+		if (mkdir(dirName.CStr(), 0700) < 0 && errno != EEXIST) {
+#else /*_ WIN32 */
+		if (mkdir(dirName.CStr()) < 0 && errno != EEXIST) {
+#endif /* _WIN32 */
+			std::cerr << "Could not create directory '" << dirName << "': Error " << errno << ", " << strerror(errno) << "\n";
+		}
+	}
 
 	bool interactive_debugger = Convert::ToBool(ScriptGlobal::Get("AttachDebugger"));
 
-	if (interactive_debugger) {
+	if (!interactive_debugger) {
 		std::ofstream ofs;
 		ofs.open(fname.CStr());
 
@@ -783,7 +807,17 @@ LONG CALLBACK Application::SEHUnhandledExceptionFilter(PEXCEPTION_POINTERS exi)
 	l_InExceptionHandler = true;
 
 	String fname = GetCrashReportFilename();
-	Utility::MkDir(Utility::DirName(fname), 0750);
+	String dirName = Utility::DirName(fname);
+
+	if (!Utility::PathExists(dirName)) {
+#ifndef _WIN32
+		if (mkdir(dirName.CStr(), 0700) < 0 && errno != EEXIST) {
+#else /*_ WIN32 */
+		if (mkdir(dirName.CStr()) < 0 && errno != EEXIST) {
+#endif /* _WIN32 */
+			std::cerr << "Could not create directory '" << dirName << "': Error " << errno << ", " << strerror(errno) << "\n";
+		}
+	}
 
 	std::ofstream ofs;
 	ofs.open(fname.CStr());
@@ -855,6 +889,8 @@ int Application::Run(void)
 		return EXIT_FAILURE;
 	}
 
+	SetMainTime(Utility::GetTime());
+
 	return Main();
 }
 
@@ -1334,6 +1370,16 @@ void Application::SetStartTime(double ts)
 	m_StartTime = ts;
 }
 
+double Application::GetMainTime(void)
+{
+	return m_MainTime;
+}
+
+void Application::SetMainTime(double ts)
+{
+	m_MainTime = ts;
+}
+
 bool Application::GetScriptDebuggerEnabled(void)
 {
 	return m_ScriptDebuggerEnabled;
@@ -1344,6 +1390,16 @@ void Application::SetScriptDebuggerEnabled(bool enabled)
 	m_ScriptDebuggerEnabled = enabled;
 }
 
+double Application::GetLastReloadFailed(void)
+{
+	return m_LastReloadFailed;
+}
+
+void Application::SetLastReloadFailed(double ts)
+{
+	m_LastReloadFailed = ts;
+}
+
 void Application::ValidateName(const String& value, const ValidationUtils& utils)
 {
 	ObjectImpl<Application>::ValidateName(value, utils);
