@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -118,7 +118,7 @@ private:
 	Object& operator=(const Object& rhs);
 
 	uintptr_t m_References;
-	mutable boost::recursive_mutex m_Mutex;
+	mutable uintptr_t m_Mutex;
 
 #ifdef I2_DEBUG
 #	ifndef _WIN32
@@ -134,8 +134,16 @@ private:
 	friend void intrusive_ptr_release(Object *object);
 };
 
+void TypeAddObject(Object *object);
+void TypeRemoveObject(Object *object);
+
 inline void intrusive_ptr_add_ref(Object *object)
 {
+#ifdef I2_LEAK_DEBUG
+	if (object->m_References == 0)
+		TypeAddObject(object);
+#endif /* I2_LEAK_DEBUG */
+
 #ifdef _WIN32
 	InterlockedIncrement(&object->m_References);
 #else /* _WIN32 */
@@ -146,14 +154,20 @@ inline void intrusive_ptr_add_ref(Object *object)
 inline void intrusive_ptr_release(Object *object)
 {
 	uintptr_t refs;
+
 #ifdef _WIN32
 	refs = InterlockedDecrement(&object->m_References);
 #else /* _WIN32 */
 	refs = __sync_sub_and_fetch(&object->m_References, 1);
 #endif /* _WIN32 */
 
-	if (refs == 0)
+	if (refs == 0) {
+#ifdef I2_LEAK_DEBUG
+		TypeRemoveObject(object);
+#endif /* I2_LEAK_DEBUG */
+
 		delete object;
+	}
 }
 
 template<typename T>
