@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -48,7 +48,7 @@ std::vector<String> icinga::GetBashCompletionSuggestions(const String& type, con
 		result.push_back(wline);
 	}
 	fclose(fp);
-	
+
 	/* Append a slash if there's only one suggestion and it's a directory */
 	if ((type == "file" || type == "directory") && result.size() == 1) {
 		String path = result[0];
@@ -73,7 +73,7 @@ std::vector<String> icinga::GetFieldCompletionSuggestions(const Type::Ptr& type,
 	for (int i = 0; i < type->GetFieldCount(); i++) {
 		Field field = type->GetFieldInfo(i);
 
-		if (!(field.Attributes & FAConfig) || field.Attributes & FAInternal)
+		if (field.Attributes & FANoUserView)
 			continue;
 
 		if (strcmp(field.TypeName, "int") != 0 && strcmp(field.TypeName, "double") != 0
@@ -200,12 +200,11 @@ bool CLICommand::ParseCommand(int argc, char **argv, po::options_description& vi
 found_command:
 	lock.unlock();
 
-	po::options_description vdesc("Command options");
-
-	if (command)
+	if (command) {
+		po::options_description vdesc("Command options");
 		command->InitParameters(vdesc, hiddenDesc);
-
-	visibleDesc.add(vdesc);
+		visibleDesc.add(vdesc);
+	}
 
 	if (autocomplete)
 		return true;
@@ -323,7 +322,7 @@ void CLICommand::ShowCommands(int argc, char **argv, po::options_description *vi
 		} else if (autoindex - 1 >= 0 && argv[autoindex - 1][0] == '-' && argv[autoindex - 1][1] != '-') {
 			aname = argv[autoindex - 1];
 			pword = aword;
-			
+
 			if (pword == "=")
 				pword = "";
 		} else if (aword.GetLength() > 1 && aword[0] == '-' && aword[1] != '-') {
@@ -349,7 +348,7 @@ void CLICommand::ShowCommands(int argc, char **argv, po::options_description *vi
 		BOOST_FOREACH(const String& suggestion, command->GetArgumentSuggestions(odesc->long_name(), pword)) {
 			std::cout << prefix << suggestion << "\n";
 		}
-		
+
 		return;
 
 complete_option:
