@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -19,6 +19,7 @@
 
 #include "cli/daemoncommand.hpp"
 #include "cli/daemonutility.hpp"
+#include "remote/apilistener.hpp"
 #include "config/configcompiler.hpp"
 #include "config/configcompilercontext.hpp"
 #include "config/configitembuilder.hpp"
@@ -35,6 +36,7 @@
 #include <boost/tuple/tuple.hpp>
 #include <boost/foreach.hpp>
 #include <iostream>
+#include <fstream>
 
 using namespace icinga;
 namespace po = boost::program_options;
@@ -152,9 +154,15 @@ static void TerminateAndWaitForEnd(pid_t target)
 		ret = kill(target, 0);
 	}
 
-	// timeout and the process still seems to live: kill it
-	if (ret == 0 || errno != ESRCH)
+	// timeout and the process still seems to live: update pid and kill it
+	if (ret == 0 || errno != ESRCH) {
+		String pidFile = Application::GetPidPath();
+		std::ofstream fp(pidFile.CStr());
+		fp << Utility::GetPid();
+		fp.close();
+
 		kill(target, SIGKILL);
+	}
 
 #else
 	// TODO: implement this for Win32
@@ -295,5 +303,7 @@ int DaemonCommand::Run(const po::variables_map& vm, const std::vector<std::strin
 	sigaction(SIGHUP, &sa, NULL);
 #endif /* _WIN32 */
 
+	ApiListener::UpdateObjectAuthority();
+
 	return Application::GetInstance()->Run();
 }
