@@ -1,21 +1,21 @@
 /******************************************************************************
-* Icinga 2                                                                   *
-* Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
-*                                                                            *
-* This program is free software; you can redistribute it and/or              *
-* modify it under the terms of the GNU General Public License                *
-* as published by the Free Software Foundation; either version 2             *
-* of the License, or (at your option) any later version.                     *
-*                                                                            *
-* This program is distributed in the hope that it will be useful,            *
-* but WITHOUT ANY WARRANTY; without even the implied warranty of             *
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
-* GNU General Public License for more details.                               *
-*                                                                            *
-* You should have received a copy of the GNU General Public License          *
-* along with this program; if not, write to the Free Software Foundation     *
-* Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.             *
-******************************************************************************/
+ * Icinga 2                                                                   *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
+ *                                                                            *
+ * This program is free software; you can redistribute it and/or              *
+ * modify it under the terms of the GNU General Public License                *
+ * as published by the Free Software Foundation; either version 2             *
+ * of the License, or (at your option) any later version.                     *
+ *                                                                            *
+ * This program is distributed in the hope that it will be useful,            *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
+ * GNU General Public License for more details.                               *
+ *                                                                            *
+ * You should have received a copy of the GNU General Public License          *
+ * along with this program; if not, write to the Free Software Foundation     *
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.             *
+ ******************************************************************************/
 
 #include "cli/daemonutility.hpp"
 #include "base/utility.hpp"
@@ -24,6 +24,7 @@
 #include "config/configcompiler.hpp"
 #include "config/configcompilercontext.hpp"
 #include "config/configitembuilder.hpp"
+#include "remote/configobjectutility.hpp"
 
 
 using namespace icinga;
@@ -60,25 +61,40 @@ static void IncludeZoneDirRecursive(const String& path, const String& package, b
 
 static void IncludeNonLocalZone(const String& zonePath, const String& package, bool& success)
 {
-	String etcPath = Application::GetZonesDir() + "/" + Utility::BaseName(zonePath);
+	/* Note: This include function must not call RegisterZoneDir().
+	 * We do not need to copy it for cluster config sync. */
 
-	if (Utility::PathExists(etcPath) || Utility::PathExists(zonePath + "/.authoritative"))
+	String zoneName = Utility::BaseName(zonePath);
+
+	/* Check whether this node already has an authoritative config version
+	 * from zones.d in etc or api package directory, or a local marker file)
+	 */
+	if (ConfigCompiler::HasZoneConfigAuthority(zoneName) || Utility::PathExists(zonePath + "/.authoritative")) {
+		Log(LogNotice, "config")
+		    << "Ignoring non local config include for zone '" << zoneName << "': We already have an authoritative copy included.";
 		return;
+	}
 
-	IncludeZoneDirRecursive(zonePath, package, success);
+	std::vector<Expression *> expressions;
+	Utility::GlobRecursive(zonePath, "*.conf", boost::bind(&ConfigCompiler::CollectIncludes, boost::ref(expressions), _1, zoneName, package), GlobFile);
+	DictExpression expr(expressions);
+	if (!ExecuteExpression(&expr))
+		success = false;
 }
 
 static void IncludePackage(const String& packagePath, bool& success)
 {
+	/* Note: Package includes will register their zones
+	 * for config sync inside their generated config. */
 	String packageName = Utility::BaseName(packagePath);
-	
+
 	if (Utility::PathExists(packagePath + "/include.conf")) {
 		Expression *expr = ConfigCompiler::CompileFile(packagePath + "/include.conf",
 		    String(), packageName);
-		
+
 		if (!ExecuteExpression(expr))
 			success = false;
-			
+
 		delete expr;
 	}
 }
@@ -99,8 +115,9 @@ bool DaemonUtility::ValidateConfigFiles(const std::vector<std::string>& configs,
 		}
 	}
 
-	/* Load cluster config files - this should probably be in libremote but
-	* unfortunately moving it there is somewhat non-trivial. */
+	/* Load cluster config files from /etc/icinga2/zones.d.
+	 * This should probably be in libremote but
+	 * unfortunately moving it there is somewhat non-trivial. */
 	success = true;
 
 	String zonesEtcDir = Application::GetZonesDir();
@@ -110,16 +127,19 @@ bool DaemonUtility::ValidateConfigFiles(const std::vector<std::string>& configs,
 	if (!success)
 		return false;
 
-	String zonesVarDir = Application::GetLocalStateDir() + "/lib/icinga2/api/zones";
-	if (Utility::PathExists(zonesVarDir))
-		Utility::Glob(zonesVarDir + "/*", boost::bind(&IncludeNonLocalZone, _1, "_cluster", boost::ref(success)), GlobDirectory);
+	/* Load package config files - they may contain additional zones which
+	 * are authoritative on this node and are checked in HasZoneConfigAuthority(). */
+	String packagesVarDir = Application::GetLocalStateDir() + "/lib/icinga2/api/packages";
+	if (Utility::PathExists(packagesVarDir))
+		Utility::Glob(packagesVarDir + "/*", boost::bind(&IncludePackage, _1, boost::ref(success)), GlobDirectory);
 
 	if (!success)
 		return false;
 
-	String packagesVarDir = Application::GetLocalStateDir() + "/lib/icinga2/api/packages";
-	if (Utility::PathExists(packagesVarDir))
-		Utility::Glob(packagesVarDir + "/*", boost::bind(&IncludePackage, _1, boost::ref(success)), GlobDirectory);
+	/* Load cluster synchronized configuration files */
+	String zonesVarDir = Application::GetLocalStateDir() + "/lib/icinga2/api/zones";
+	if (Utility::PathExists(zonesVarDir))
+		Utility::Glob(zonesVarDir + "/*", boost::bind(&IncludeNonLocalZone, _1, "_cluster", boost::ref(success)), GlobDirectory);
 
 	if (!success)
 		return false;
@@ -149,6 +169,9 @@ bool DaemonUtility::LoadConfigFiles(const std::vector<std::string>& configs,
 	WorkQueue upq(25000, Application::GetConcurrency());
 	bool result = ConfigItem::CommitItems(ascope.GetContext(), upq, newItems);
 
+	/* Remove ignored Downtime/Comment objects. */
+	ConfigItem::RemoveIgnoredItems(ConfigObjectUtility::GetConfigDir());
+
 	if (!result)
 		return false;
 
