@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -166,7 +166,7 @@ void NodeUtility::AddNode(const String& name)
 	node->Set("repository", Empty);
 
 	CreateRepositoryPath();
-	Utility::SaveJsonFile(path, node);
+	Utility::SaveJsonFile(path, 0600, node);
 }
 
 void NodeUtility::AddNodeSettings(const String& name, const String& host,
@@ -179,7 +179,7 @@ void NodeUtility::AddNodeSettings(const String& name, const String& host,
 	settings->Set("log_duration", log_duration);
 
 	CreateRepositoryPath();
-	Utility::SaveJsonFile(GetNodeSettingsFile(name), settings);
+	Utility::SaveJsonFile(GetNodeSettingsFile(name), 0600, settings);
 }
 
 void NodeUtility::RemoveNode(const String& name)
@@ -385,9 +385,8 @@ bool NodeUtility::WriteNodeConfigObjects(const String& filename, const Array::Pt
 		    << "Cannot set ownership for user '" << user << "' group '" << group << "' on path '" << path << "'. Verify it yourself!";
 	}
 
-	String tempPath = filename + ".tmp";
-
-	std::ofstream fp(tempPath.CStr(), std::ofstream::out | std::ostream::trunc);
+	std::fstream fp;
+	String tempFilename = Utility::CreateTempFile(filename + ".XXXXXX", 0644, fp);
 
 	fp << "/*\n";
 	fp << " * Generated by Icinga 2 node setup commands\n";
@@ -406,11 +405,11 @@ bool NodeUtility::WriteNodeConfigObjects(const String& filename, const Array::Pt
 	_unlink(filename.CStr());
 #endif /* _WIN32 */
 
-	if (rename(tempPath.CStr(), filename.CStr()) < 0) {
+	if (rename(tempFilename.CStr(), filename.CStr()) < 0) {
 		BOOST_THROW_EXCEPTION(posix_error()
 		    << boost::errinfo_api_function("rename")
 		    << boost::errinfo_errno(errno)
-		    << boost::errinfo_file_name(tempPath));
+		    << boost::errinfo_file_name(tempFilename));
 	}
 
 	return true;
@@ -471,7 +470,7 @@ int NodeUtility::UpdateBlackAndWhiteList(const String& type, const String& zone_
 
 	String list_path = GetBlackAndWhiteListPath(type);
 	CreateRepositoryPath();
-	Utility::SaveJsonFile(list_path, lists);
+	Utility::SaveJsonFile(list_path, 0600, lists);
 
 	return 0;
 }
@@ -516,7 +515,7 @@ int NodeUtility::RemoveBlackAndWhiteList(const String& type, const String& zone_
 
 	String list_path = GetBlackAndWhiteListPath(type);
 	CreateRepositoryPath();
-	Utility::SaveJsonFile(list_path, lists);
+	Utility::SaveJsonFile(list_path, 0600, lists);
 
 	return 0;
 }
@@ -629,7 +628,7 @@ void NodeUtility::SerializeObject(std::ostream& fp, const Dictionary::Ptr& objec
 		ConfigWriter::EmitIdentifier(fp, kv.first, true);
 		fp << " = ";
 		ConfigWriter::EmitValue(fp, 1, kv.second);
-		fp << ";\n";
+		fp << "\n";
 	}
 
 	fp << "}\n\n";
@@ -638,10 +637,10 @@ void NodeUtility::SerializeObject(std::ostream& fp, const Dictionary::Ptr& objec
 void NodeUtility::UpdateConstant(const String& name, const String& value)
 {
 	String constantsFile = Application::GetSysconfDir() + "/icinga2/constants.conf";
-	String tempFile = constantsFile + ".tmp";
 
 	std::ifstream ifp(constantsFile.CStr());
-	std::ofstream ofp(tempFile.CStr());
+	std::fstream ofp;
+	String tempFile = Utility::CreateTempFile(constantsFile + ".XXXXXX", 0644, ofp);
 
 	bool found = false;
 
