@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -22,6 +22,7 @@
 #include "base/logger.hpp"
 #include "base/application.hpp"
 #include "base/convert.hpp"
+#include "base/configwriter.hpp"
 #include "base/scriptglobal.hpp"
 #include "base/json.hpp"
 #include "base/netstring.hpp"
@@ -233,12 +234,15 @@ bool RepositoryUtility::AddObject(const std::vector<String>& object_paths, const
 	if (check_config) {
 		try {
 			ConfigObject::Ptr object = static_pointer_cast<ConfigObject>(utype->Instantiate());
+			/* temporarly set the object type for validation */
 			attrs->Set("type", utype->GetName());
 			Deserialize(object, attrs, false, FAConfig);
 			object->SetName(name);
 
 			RepositoryValidationUtils utils;
 			static_pointer_cast<ConfigObject>(object)->Validate(FAConfig, utils);
+
+			attrs->Remove("type");
 		} catch (const ValidationError& ex) {
 			Log(LogCritical, "config", DiagnosticInformation(ex));
 			return false;
@@ -352,9 +356,9 @@ bool RepositoryUtility::WriteObjectToRepositoryChangeLog(const String& path, con
 
 	CreateRepositoryPath(Utility::DirName(path));
 
-	String tempPath = path + ".tmp";
+	std::fstream fp;
+	String tempFilename = Utility::CreateTempFile(path + ".XXXXXX", 0600, fp);
 
-	std::ofstream fp(tempPath.CStr(), std::ofstream::out | std::ostream::trunc);
 	fp << JsonEncode(item);
 	fp.close();
 
@@ -362,11 +366,11 @@ bool RepositoryUtility::WriteObjectToRepositoryChangeLog(const String& path, con
 	_unlink(path.CStr());
 #endif /* _WIN32 */
 
-	if (rename(tempPath.CStr(), path.CStr()) < 0) {
+	if (rename(tempFilename.CStr(), path.CStr()) < 0) {
 		BOOST_THROW_EXCEPTION(posix_error()
 		    << boost::errinfo_api_function("rename")
 		    << boost::errinfo_errno(errno)
-		    << boost::errinfo_file_name(tempPath));
+		    << boost::errinfo_file_name(tempFilename));
 	}
 
 	return true;
@@ -492,9 +496,9 @@ bool RepositoryUtility::WriteObjectToRepository(const String& path, const String
 
 	CreateRepositoryPath(Utility::DirName(path));
 
-	String tempPath = path + ".tmp";
+	std::fstream fp;
+	String tempFilename = Utility::CreateTempFile(path + ".XXXXXX", 0644, fp);
 
-	std::ofstream fp(tempPath.CStr(), std::ofstream::out | std::ostream::trunc);
 	SerializeObject(fp, name, type, item);
 	fp << std::endl;
 	fp.close();
@@ -503,11 +507,11 @@ bool RepositoryUtility::WriteObjectToRepository(const String& path, const String
 	_unlink(path.CStr());
 #endif /* _WIN32 */
 
-	if (rename(tempPath.CStr(), path.CStr()) < 0) {
+	if (rename(tempFilename.CStr(), path.CStr()) < 0) {
 		BOOST_THROW_EXCEPTION(posix_error()
 		    << boost::errinfo_api_function("rename")
 		    << boost::errinfo_errno(errno)
-		    << boost::errinfo_file_name(tempPath));
+		    << boost::errinfo_file_name(tempFilename));
 	}
 
 	return true;
@@ -608,18 +612,15 @@ void RepositoryUtility::CommitChange(const Dictionary::Ptr& change, const String
 	String command = change->Get("command");
 	Dictionary::Ptr attrs;
 
-	if (change->Contains("attrs")) {
+	if (change->Contains("attrs"))
 		attrs = change->Get("attrs");
-	}
 
 	bool success = false;
 
-	if (command == "add") {
+	if (command == "add")
 		success = AddObjectInternal(name, type, attrs);
-	}
-	else if (command == "remove") {
+	else if (command == "remove")
 		success = RemoveObjectInternal(name, type, attrs);
-	}
 
 	if (success) {
 		Log(LogNotice, "cli")
@@ -669,7 +670,7 @@ void RepositoryUtility::FormatChangelogEntry(std::ostream& fp, const Dictionary:
 			continue;
 
 		fp << std::setw(4) << " " << ConsoleColorTag(Console_ForegroundGreen) << kv.first << ConsoleColorTag(Console_Normal) << " = ";
-		FormatValue(fp, kv.second);
+		ConfigWriter::EmitValue(fp, 0, kv.second);
 		fp << "\n";
 	}
 }
@@ -680,7 +681,9 @@ void RepositoryUtility::FormatChangelogEntry(std::ostream& fp, const Dictionary:
  */
 void RepositoryUtility::SerializeObject(std::ostream& fp, const String& name, const String& type, const Dictionary::Ptr& object)
 {
-	fp << "object " << type << " \"" << name << "\" {\n";
+	fp << "object " << type << " ";
+	ConfigWriter::EmitString(fp, name);
+ 	fp << " {\n";
 
 	if (!object) {
 		fp << "}\n";
@@ -692,7 +695,9 @@ void RepositoryUtility::SerializeObject(std::ostream& fp, const String& name, co
 
 		ObjectLock olock(imports);
 		BOOST_FOREACH(const String& import, imports) {
-			fp << "\t" << "import \"" << import << "\"\n";
+			fp << "\t" << "import ";
+			ConfigWriter::EmitString(fp, import);
+			fp << '\n';
 		}
 	}
 
@@ -701,49 +706,12 @@ void RepositoryUtility::SerializeObject(std::ostream& fp, const String& name, co
 		if (kv.first == "import" || kv.first == "name" || kv.first == "__name") {
 			continue;
 		} else {
-			fp << "\t" << kv.first << " = ";
-			FormatValue(fp, kv.second);
+			fp << "\t";
+			ConfigWriter::EmitIdentifier(fp, kv.first, true);
+			fp << " = ";
+			ConfigWriter::EmitValue(fp, 1, kv.second);
 		}
 		fp << "\n";
 	}
 	fp << "}\n";
 }
-
-void RepositoryUtility::FormatValue(std::ostream& fp, const Value& val)
-{
-	if (val.IsObjectType<Array>()) {
-		FormatArray(fp, val);
-		return;
-	}
-
-	if (val.IsString()) {
-		fp << "\"" << Convert::ToString(val) << "\"";
-		return;
-	}
-
-	fp << Convert::ToString(val);
-}
-
-void RepositoryUtility::FormatArray(std::ostream& fp, const Array::Ptr& arr)
-{
-	bool first = true;
-
-	fp << "[ ";
-
-	if (arr) {
-		ObjectLock olock(arr);
-		BOOST_FOREACH(const Value& value, arr) {
-			if (first)
-				first = false;
-			else
-				fp << ", ";
-
-			FormatValue(fp, value);
-		}
-	}
-
-	if (!first)
-		fp << " ";
-
-	fp << "]";
-}
