@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -75,6 +75,8 @@ public:
 
 	static std::vector<ConfigItem::Ptr> GetItems(const String& type);
 
+	static void RemoveIgnoredItems(const String& allowedConfigPath);
+
 private:
 	String m_Type; /**< The object type. */
 	String m_Name; /**< The name. */
@@ -99,7 +101,9 @@ private:
 
 	typedef std::vector<ConfigItem::Ptr> ItemList;
 	static ItemList m_UnnamedItems;
-	static ItemList m_CommittedItems;
+
+	typedef std::vector<String> IgnoredItemList;
+	static IgnoredItemList m_IgnoredItems;
 
 	static ConfigItem::Ptr GetObjectUnlocked(const String& type,
 	    const String& name);
