@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -57,9 +57,6 @@ public:
 	DbReference GetInsertID(const DbObject::Ptr& dbobj) const;
 	DbReference GetInsertID(const DbType::Ptr& type, const DbReference& objid) const;
 
-	void SetNotificationInsertID(const CustomVarObject::Ptr& obj, const DbReference& dbref);
-	DbReference GetNotificationInsertID(const CustomVarObject::Ptr& obj) const;
-
 	void SetObjectActive(const DbObject::Ptr& dbobj, bool active);
 	bool GetObjectActive(const DbObject::Ptr& dbobj) const;
 
@@ -83,6 +80,7 @@ protected:
 	virtual void Pause(void) override;
 
 	virtual void ExecuteQuery(const DbQuery& query) = 0;
+	virtual void ExecuteMultipleQueries(const std::vector<DbQuery>&) = 0;
 	virtual void ActivateObject(const DbObject::Ptr& dbobj) = 0;
 	virtual void DeactivateObject(const DbObject::Ptr& dbobj) = 0;
 
@@ -97,10 +95,11 @@ protected:
 
 	void IncreaseQueryCount(void);
 
+	static void UpdateProgramStatus(void);
+
 private:
 	std::map<DbObject::Ptr, DbReference> m_ObjectIDs;
 	std::map<std::pair<DbType::Ptr, DbReference>, DbReference> m_InsertIDs;
-	std::map<CustomVarObject::Ptr, DbReference> m_NotificationInsertIDs;
 	std::set<DbObject::Ptr> m_ActiveObjects;
 	std::set<DbObject::Ptr> m_ConfigUpdates;
 	std::set<DbObject::Ptr> m_StatusUpdates;
@@ -113,11 +112,15 @@ private:
 	static Timer::Ptr m_ProgramStatusTimer;
 	static boost::once_flag m_OnceFlag;
 
+	Timer::Ptr m_StatsLoggerTimer;
+	void StatsLoggerTimerHandler(void);
+
 	static void InsertRuntimeVariable(const String& key, const Value& value);
-	static void ProgramStatusHandler(void);
 
 	mutable boost::mutex m_StatsMutex;
 	RingBuffer m_QueryStats;
+	int m_PendingQueries;
+	double m_PendingQueriesTimestamp;
 };
 
 struct database_error : virtual std::exception, virtual boost::exception { };
