@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -45,13 +45,13 @@ void DbEvents::StaticInitialize(void)
 	/* Status */
 	Comment::OnCommentAdded.connect(boost::bind(&DbEvents::AddComment, _1));
 	Comment::OnCommentRemoved.connect(boost::bind(&DbEvents::RemoveComment, _1));
-	Downtime::OnDowntimeAdded.connect(boost::bind(&DbEvents::AddDowntime, _1, true));
+	Downtime::OnDowntimeAdded.connect(boost::bind(&DbEvents::AddDowntime, _1));
 	Downtime::OnDowntimeRemoved.connect(boost::bind(&DbEvents::RemoveDowntime, _1));
 	Downtime::OnDowntimeTriggered.connect(boost::bind(&DbEvents::TriggerDowntime, _1));
 	Checkable::OnAcknowledgementSet.connect(boost::bind(&DbEvents::AddAcknowledgement, _1, _4));
 	Checkable::OnAcknowledgementCleared.connect(boost::bind(&DbEvents::RemoveAcknowledgement, _1));
 
-	Checkable::OnNextCheckChanged.connect(boost::bind(&DbEvents::NextCheckChangedHandler, _1));
+	Checkable::OnNextCheckUpdated.connect(boost::bind(&DbEvents::NextCheckUpdatedHandler, _1));
 	Checkable::OnFlappingChanged.connect(boost::bind(&DbEvents::FlappingChangedHandler, _1));
 	Checkable::OnNotificationSentToAllUsers.connect(boost::bind(&DbEvents::LastNotificationChangedHandler, _1, _2));
 
@@ -89,7 +89,7 @@ void DbEvents::StaticInitialize(void)
 }
 
 /* check events */
-void DbEvents::NextCheckChangedHandler(const Checkable::Ptr& checkable)
+void DbEvents::NextCheckUpdatedHandler(const Checkable::Ptr& checkable)
 {
 	Host::Ptr host;
 	Service::Ptr service;
@@ -101,7 +101,7 @@ void DbEvents::NextCheckChangedHandler(const Checkable::Ptr& checkable)
 	else
 		query1.Table = "hoststatus";
 
-	query1.Type = DbQueryInsert | DbQueryUpdate;
+	query1.Type = DbQueryUpdate;
 	query1.Category = DbCatState;
 	query1.StatusUpdate = true;
 	query1.Object = DbObject::GetOrCreateByObject(checkable);
@@ -117,8 +117,6 @@ void DbEvents::NextCheckChangedHandler(const Checkable::Ptr& checkable)
 	else
 		query1.WhereCriteria->Set("host_object_id", host);
 
-	query1.WhereCriteria->Set("instance_id", 0); /* DbConnection class fills in real ID */
-
 	DbObject::OnQuery(query1);
 }
 
@@ -134,7 +132,7 @@ void DbEvents::FlappingChangedHandler(const Checkable::Ptr& checkable)
 	else
 		query1.Table = "hoststatus";
 
-	query1.Type = DbQueryInsert | DbQueryUpdate;
+	query1.Type = DbQueryUpdate;
 	query1.Category = DbCatState;
 	query1.StatusUpdate = true;
 	query1.Object = DbObject::GetOrCreateByObject(checkable);
@@ -172,7 +170,7 @@ void DbEvents::LastNotificationChangedHandler(const Notification::Ptr& notificat
 	else
 		query1.Table = "hoststatus";
 
-	query1.Type = DbQueryInsert | DbQueryUpdate;
+	query1.Type = DbQueryUpdate;
 	query1.Category = DbCatState;
 	query1.StatusUpdate = true;
 	query1.Object = DbObject::GetOrCreateByObject(checkable);
@@ -219,7 +217,7 @@ void DbEvents::ReachabilityChangedHandler(const Checkable::Ptr& checkable, const
 		else
 			query1.Table = "hoststatus";
 
-		query1.Type = DbQueryInsert | DbQueryUpdate;
+		query1.Type = DbQueryUpdate;
 		query1.Category = DbCatState;
 		query1.StatusUpdate = true;
 		query1.Object = DbObject::GetOrCreateByObject(child);
@@ -279,7 +277,7 @@ void DbEvents::EnableChangedHandlerInternal(const Checkable::Ptr& checkable, con
 	else
 		query1.Table = "hoststatus";
 
-	query1.Type = DbQueryInsert | DbQueryUpdate;
+	query1.Type = DbQueryUpdate;
 	query1.Category = DbCatState;
 	query1.StatusUpdate = true;
 	query1.Object = DbObject::GetOrCreateByObject(checkable);
@@ -305,30 +303,42 @@ void DbEvents::AddComments(const Checkable::Ptr& checkable)
 {
 	std::set<Comment::Ptr> comments = checkable->GetComments();
 
-	if (!comments.empty())
-		RemoveComments(checkable);
+	if (comments.empty())
+		return;
+
+	std::vector<DbQuery> queries;
+
+	DbQuery query1;
+	query1.Table = "comments";
+	query1.Type = DbQueryDelete;
+	query1.Category = DbCatComment;
+	query1.WhereCriteria = new Dictionary();
+	query1.WhereCriteria->Set("object_id", checkable);
+	queries.push_back(query1);
 
 	BOOST_FOREACH(const Comment::Ptr& comment, comments) {
-		AddComment(comment);
+		AddCommentInternal(queries, comment, false);
 	}
+
+	DbObject::OnMultipleQueries(queries);
 }
 
 void DbEvents::AddComment(const Comment::Ptr& comment)
 {
-	AddCommentInternal(comment, false);
+	std::vector<DbQuery> queries;
+	RemoveCommentInternal(queries, comment);
+	AddCommentInternal(queries, comment, false);
+	DbObject::OnMultipleQueries(queries);
 }
 
 void DbEvents::AddCommentHistory(const Comment::Ptr& comment)
 {
-	AddCommentInternal(comment, true);
+	std::vector<DbQuery> queries;
+	AddCommentInternal(queries, comment, true);
+	DbObject::OnMultipleQueries(queries);
 }
 
-void DbEvents::AddCommentInternal(const Comment::Ptr& comment, bool historical)
-{
-	AddCommentByType(comment, historical);
-}
-
-void DbEvents::AddCommentByType(const Comment::Ptr& comment, bool historical)
+void DbEvents::AddCommentInternal(std::vector<DbQuery>& queries, const Comment::Ptr& comment, bool historical)
 {
 	Checkable::Ptr checkable = comment->GetCheckable();
 
@@ -378,24 +388,18 @@ void DbEvents::AddCommentByType(const Comment::Ptr& comment, bool historical)
 	query1.Type = DbQueryInsert;
 	query1.Category = DbCatComment;
 	query1.Fields = fields1;
-	DbObject::OnQuery(query1);
-}
 
-void DbEvents::RemoveComments(const Checkable::Ptr& checkable)
-{
-	Log(LogDebug, "DbEvents")
-	    << "removing service comments for '" << checkable->GetName() << "'";
-
-	DbQuery query1;
-	query1.Table = "comments";
-	query1.Type = DbQueryDelete;
-	query1.Category = DbCatComment;
-	query1.WhereCriteria = new Dictionary();
-	query1.WhereCriteria->Set("object_id", checkable);
-	DbObject::OnQuery(query1);
+	queries.push_back(query1);
 }
 
 void DbEvents::RemoveComment(const Comment::Ptr& comment)
+{
+	std::vector<DbQuery> queries;
+	RemoveCommentInternal(queries, comment);
+	DbObject::OnMultipleQueries(queries);
+}
+
+void DbEvents::RemoveCommentInternal(std::vector<DbQuery>& queries, const Comment::Ptr& comment)
 {
 	Checkable::Ptr checkable = comment->GetCheckable();
 
@@ -407,7 +411,7 @@ void DbEvents::RemoveComment(const Comment::Ptr& comment)
 	query1.WhereCriteria = new Dictionary();
 	query1.WhereCriteria->Set("object_id", checkable);
 	query1.WhereCriteria->Set("internal_comment_id", comment->GetLegacyId());
-	DbObject::OnQuery(query1);
+	queries.push_back(query1);
 
 	/* History - update deletion time for service/host */
 	unsigned long entry_time = static_cast<long>(comment->GetEntryTime());
@@ -427,10 +431,10 @@ void DbEvents::RemoveComment(const Comment::Ptr& comment)
 
 	query2.WhereCriteria = new Dictionary();
 	query2.WhereCriteria->Set("internal_comment_id", comment->GetLegacyId());
+	query2.WhereCriteria->Set("object_id", checkable);
 	query2.WhereCriteria->Set("comment_time", DbValue::FromTimestamp(entry_time));
 	query2.WhereCriteria->Set("instance_id", 0); /* DbConnection class fills in real ID */
-
-	DbObject::OnQuery(query2);
+	queries.push_back(query2);
 }
 
 /* downtimes */
@@ -438,37 +442,42 @@ void DbEvents::AddDowntimes(const Checkable::Ptr& checkable)
 {
 	std::set<Downtime::Ptr> downtimes = checkable->GetDowntimes();
 
-	if (!downtimes.empty())
-		RemoveDowntimes(checkable);
+	if (downtimes.empty())
+		return;
+
+	std::vector<DbQuery> queries;
+
+	DbQuery query1;
+	query1.Table = "scheduleddowntime";
+	query1.Type = DbQueryDelete;
+	query1.Category = DbCatDowntime;
+	query1.WhereCriteria = new Dictionary();
+	query1.WhereCriteria->Set("object_id", checkable);
+	queries.push_back(query1);
 
 	BOOST_FOREACH(const Downtime::Ptr& downtime, downtimes) {
-		AddDowntime(downtime, false);
+		AddDowntimeInternal(queries, downtime, false);
 	}
+
+	DbObject::OnMultipleQueries(queries);
 }
 
-void DbEvents::AddDowntime(const Downtime::Ptr& downtime, bool remove_existing)
+void DbEvents::AddDowntime(const Downtime::Ptr& downtime)
 {
-	/*
-	 * make sure to delete any old downtime to avoid multiple inserts from
-	 * configured ScheduledDowntime dumps and CreateNextDowntime() calls
-	 */
-	if (remove_existing)
-		RemoveDowntime(downtime);
-
-	AddDowntimeInternal(downtime, false);
+	std::vector<DbQuery> queries;
+	RemoveDowntimeInternal(queries, downtime);
+	AddDowntimeInternal(queries, downtime, false);
+	DbObject::OnMultipleQueries(queries);
 }
 
 void DbEvents::AddDowntimeHistory(const Downtime::Ptr& downtime)
 {
-	AddDowntimeInternal(downtime, true);
+	std::vector<DbQuery> queries;
+	AddDowntimeInternal(queries, downtime, true);
+	DbObject::OnMultipleQueries(queries);
 }
 
-void DbEvents::AddDowntimeInternal(const Downtime::Ptr& downtime, bool historical)
-{
-	AddDowntimeByType(downtime, historical);
-}
-
-void DbEvents::AddDowntimeByType(const Downtime::Ptr& downtime, bool historical)
+void DbEvents::AddDowntimeInternal(std::vector<DbQuery>& queries, const Downtime::Ptr& downtime, bool historical)
 {
 	Checkable::Ptr checkable = downtime->GetCheckable();
 
@@ -496,8 +505,9 @@ void DbEvents::AddDowntimeByType(const Downtime::Ptr& downtime, bool historical)
 	fields1->Set("duration", downtime->GetDuration());
 	fields1->Set("scheduled_start_time", DbValue::FromTimestamp(downtime->GetStartTime()));
 	fields1->Set("scheduled_end_time", DbValue::FromTimestamp(downtime->GetEndTime()));
-	fields1->Set("was_started", 0);
-	fields1->Set("is_in_effect", 0);
+	fields1->Set("was_started", ((downtime->GetStartTime() <= Utility::GetTime()) ? 1 : 0));
+	fields1->Set("is_in_effect", (downtime->IsInEffect() ? 1 : 0));
+	fields1->Set("trigger_time", DbValue::FromTimestamp(downtime->GetTriggerTime()));
 	fields1->Set("instance_id", 0); /* DbConnection class fills in real ID */
 
 	String node = IcingaApplication::GetInstance()->GetNodeName();
@@ -516,21 +526,51 @@ void DbEvents::AddDowntimeByType(const Downtime::Ptr& downtime, bool historical)
 	query1.Type = DbQueryInsert;
 	query1.Category = DbCatDowntime;
 	query1.Fields = fields1;
-	DbObject::OnQuery(query1);
-}
 
-void DbEvents::RemoveDowntimes(const Checkable::Ptr& checkable)
-{
-	DbQuery query1;
-	query1.Table = "scheduleddowntime";
-	query1.Type = DbQueryDelete;
-	query1.Category = DbCatDowntime;
-	query1.WhereCriteria = new Dictionary();
-	query1.WhereCriteria->Set("object_id", checkable);
-	DbObject::OnQuery(query1);
+	queries.push_back(query1);
+
+	/* host/service status */
+	if (!historical) {
+		Host::Ptr host;
+		Service::Ptr service;
+		tie(host, service) = GetHostService(checkable);
+
+		DbQuery query2;
+		if (service)
+			query2.Table = "servicestatus";
+		else
+			query2.Table = "hoststatus";
+
+		query2.Type = DbQueryUpdate;
+		query2.Category = DbCatState;
+		query2.StatusUpdate = true;
+		query2.Object = DbObject::GetOrCreateByObject(checkable);
+
+		Dictionary::Ptr fields2 = new Dictionary();
+		fields2->Set("scheduled_downtime_depth", checkable->GetDowntimeDepth());
+
+		query2.Fields = fields2;
+
+		query2.WhereCriteria = new Dictionary();
+		if (service)
+			query2.WhereCriteria->Set("service_object_id", service);
+		else
+			query2.WhereCriteria->Set("host_object_id", host);
+
+		query2.WhereCriteria->Set("instance_id", 0); /* DbConnection class fills in real ID */
+
+		queries.push_back(query2);
+	}
 }
 
 void DbEvents::RemoveDowntime(const Downtime::Ptr& downtime)
+{
+	std::vector<DbQuery> queries;
+	RemoveDowntimeInternal(queries, downtime);
+	DbObject::OnMultipleQueries(queries);
+}
+
+void DbEvents::RemoveDowntimeInternal(std::vector<DbQuery>& queries, const Downtime::Ptr& downtime)
 {
 	Checkable::Ptr checkable = downtime->GetCheckable();
 
@@ -543,7 +583,7 @@ void DbEvents::RemoveDowntime(const Downtime::Ptr& downtime)
 	query1.WhereCriteria->Set("object_id", checkable);
 	query1.WhereCriteria->Set("internal_downtime_id", downtime->GetLegacyId());
 	query1.WhereCriteria->Set("instance_id", 0); /* DbConnection class fills in real ID */
-	DbObject::OnQuery(query1);
+	queries.push_back(query1);
 
 	/* History - update actual_end_time, was_cancelled for service (and host in case) */
 	double now = Utility::GetTime();
@@ -558,6 +598,7 @@ void DbEvents::RemoveDowntime(const Downtime::Ptr& downtime)
 	fields3->Set("was_cancelled", downtime->GetWasCancelled() ? 1 : 0);
 	fields3->Set("actual_end_time", DbValue::FromTimestamp(time_bag.first));
 	fields3->Set("actual_end_time_usec", time_bag.second);
+	fields3->Set("is_in_effect", 0);
 	query3.Fields = fields3;
 
 	query3.WhereCriteria = new Dictionary();
@@ -566,7 +607,7 @@ void DbEvents::RemoveDowntime(const Downtime::Ptr& downtime)
 	query3.WhereCriteria->Set("entry_time", DbValue::FromTimestamp(downtime->GetEntryTime()));
 	query3.WhereCriteria->Set("instance_id", 0); /* DbConnection class fills in real ID */
 
-	DbObject::OnQuery(query3);
+	queries.push_back(query3);
 
 	/* host/service status */
 	Host::Ptr host;
@@ -579,7 +620,7 @@ void DbEvents::RemoveDowntime(const Downtime::Ptr& downtime)
 	else
 		query4.Table = "hoststatus";
 
-	query4.Type = DbQueryInsert | DbQueryUpdate;
+	query4.Type = DbQueryUpdate;
 	query4.Category = DbCatState;
 	query4.StatusUpdate = true;
 	query4.Object = DbObject::GetOrCreateByObject(checkable);
@@ -597,7 +638,7 @@ void DbEvents::RemoveDowntime(const Downtime::Ptr& downtime)
 
 	query4.WhereCriteria->Set("instance_id", 0); /* DbConnection class fills in real ID */
 
-	DbObject::OnQuery(query4);
+	queries.push_back(query4);
 }
 
 void DbEvents::TriggerDowntime(const Downtime::Ptr& downtime)
@@ -617,7 +658,7 @@ void DbEvents::TriggerDowntime(const Downtime::Ptr& downtime)
 	fields1->Set("was_started", 1);
 	fields1->Set("actual_start_time", DbValue::FromTimestamp(time_bag.first));
 	fields1->Set("actual_start_time_usec", time_bag.second);
-	fields1->Set("is_in_effect", 1);
+	fields1->Set("is_in_effect", (downtime->IsInEffect() ? 1 : 0));
 	fields1->Set("trigger_time", DbValue::FromTimestamp(downtime->GetTriggerTime()));
 	fields1->Set("instance_id", 0); /* DbConnection class fills in real ID */
 
@@ -662,7 +703,7 @@ void DbEvents::TriggerDowntime(const Downtime::Ptr& downtime)
 	else
 		query4.Table = "hoststatus";
 
-	query4.Type = DbQueryInsert | DbQueryUpdate;
+	query4.Type = DbQueryUpdate;
 	query4.Category = DbCatState;
 	query4.StatusUpdate = true;
 	query4.Object = DbObject::GetOrCreateByObject(checkable);
@@ -756,7 +797,7 @@ void DbEvents::AddAcknowledgementInternal(const Checkable::Ptr& checkable, Ackno
 	else
 		query1.Table = "hoststatus";
 
-	query1.Type = DbQueryInsert | DbQueryUpdate;
+	query1.Type = DbQueryUpdate;
 	query1.Category = DbCatState;
 	query1.StatusUpdate = true;
 	query1.Object = DbObject::GetOrCreateByObject(checkable);
@@ -792,8 +833,7 @@ void DbEvents::AddNotificationHistory(const Notification::Ptr& notification, con
 	query1.Table = "notifications";
 	query1.Type = DbQueryInsert;
 	query1.Category = DbCatNotification;
-	/* store the object ptr for caching the insert id for this object */
-	query1.NotificationObject = notification;
+	query1.NotificationInsertID = new DbValue(DbValueObjectInsertID, -1);
 
 	Host::Ptr host;
 	Service::Ptr service;
@@ -827,16 +867,17 @@ void DbEvents::AddNotificationHistory(const Notification::Ptr& notification, con
 	query1.Fields = fields1;
 	DbObject::OnQuery(query1);
 
-	DbQuery query2;
-	query2.Table = "contactnotifications";
-	query2.Type = DbQueryInsert;
-	query2.Category = DbCatNotification;
+	std::vector<DbQuery> queries;
 
-	/* filtered users */
 	BOOST_FOREACH(const User::Ptr& user, users) {
 		Log(LogDebug, "DbEvents")
 		    << "add contact notification history for service '" << checkable->GetName() << "' and user '" << user->GetName() << "'.";
 
+		DbQuery query2;
+		query2.Table = "contactnotifications";
+		query2.Type = DbQueryInsert;
+		query2.Category = DbCatNotification;
+
 		Dictionary::Ptr fields2 = new Dictionary();
 		fields2->Set("contact_object_id", user);
 		fields2->Set("start_time", DbValue::FromTimestamp(time_bag.first));
@@ -844,12 +885,14 @@ void DbEvents::AddNotificationHistory(const Notification::Ptr& notification, con
 		fields2->Set("end_time", DbValue::FromTimestamp(time_bag.first));
 		fields2->Set("end_time_usec", time_bag.second);
 
-		fields2->Set("notification_id", notification); /* DbConnection class fills in real ID from notification insert id cache */
+		fields2->Set("notification_id", query1.NotificationInsertID);
 		fields2->Set("instance_id", 0); /* DbConnection class fills in real ID */
 
 		query2.Fields = fields2;
-		DbObject::OnQuery(query2);
+		queries.push_back(query2);
 	}
+
+	DbObject::OnMultipleQueries(queries);
 }
 
 /* statehistory */
@@ -1348,7 +1391,7 @@ void DbEvents::AddCheckableCheckHistory(const Checkable::Ptr& checkable, const C
 	double end = cr->GetExecutionEnd();
 	std::pair<unsigned long, unsigned long> time_bag_end = CompatUtility::ConvertTimestamp(end);
 
-	double execution_time = Service::CalculateExecutionTime(cr);
+	double execution_time = cr->CalculateExecutionTime();
 
 	fields1->Set("start_time", DbValue::FromTimestamp(time_bag_start.first));
 	fields1->Set("start_time_usec", time_bag_start.second);
@@ -1358,7 +1401,7 @@ void DbEvents::AddCheckableCheckHistory(const Checkable::Ptr& checkable, const C
 	fields1->Set("command_args", Empty);
 	fields1->Set("command_line", CompatUtility::GetCommandLine(checkable->GetCheckCommand()));
 	fields1->Set("execution_time", Convert::ToString(execution_time));
-	fields1->Set("latency", Convert::ToString(Service::CalculateLatency(cr)));
+	fields1->Set("latency", Convert::ToString(cr->CalculateLatency()));
 	fields1->Set("return_code", cr->GetExitStatus());
 	fields1->Set("output", CompatUtility::GetCheckResultOutput(cr));
 	fields1->Set("long_output", CompatUtility::GetCheckResultLongOutput(cr));
