@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -37,6 +37,7 @@
 using namespace icinga;
 
 boost::signals2::signal<void (const DbQuery&)> DbObject::OnQuery;
+boost::signals2::signal<void (const std::vector<DbQuery>&)> DbObject::OnMultipleQueries;
 
 INITIALIZE_ONCE(&DbObject::StaticInitialize);
 
@@ -129,9 +130,6 @@ void DbObject::SendStatusUpdate(void)
 	if (query.Table != "endpointstatus" && query.Table != "zonestatus") {
 		String node = IcingaApplication::GetInstance()->GetNodeName();
 
-		Log(LogDebug, "DbObject")
-		    << "Endpoint node: '" << node << "' status update for '" << GetObject()->GetName() << "'";
-
 		Endpoint::Ptr endpoint = Endpoint::GetByName(node);
 		if (endpoint)
 			query.Fields->Set("endpoint_object_id", endpoint);
@@ -163,9 +161,6 @@ void DbObject::SendVarsConfigUpdate(void)
 	Dictionary::Ptr vars = CompatUtility::GetCustomAttributeConfig(custom_var_object);
 
 	if (vars) {
-		Log(LogDebug, "DbObject")
-		    << "Updating object vars for '" << custom_var_object->GetName() << "'";
-
 		ObjectLock olock (vars);
 
 		BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
@@ -218,9 +213,6 @@ void DbObject::SendVarsStatusUpdate(void)
 	Dictionary::Ptr vars = CompatUtility::GetCustomAttributeConfig(custom_var_object);
 
 	if (vars) {
-		Log(LogDebug, "DbObject")
-		    << "Updating object vars for '" << custom_var_object->GetName() << "'";
-
 		ObjectLock olock (vars);
 
 		BOOST_FOREACH(const Dictionary::Pair& kv, vars) {
@@ -343,9 +335,6 @@ void DbObject::VarsChangedHandler(const CustomVarObject::Ptr& object)
 {
 	DbObject::Ptr dbobj = GetOrCreateByObject(object);
 
-	Log(LogDebug, "DbObject")
-	    << "Vars changed for object '" << object->GetName() << "'";
-
 	if (!dbobj)
 		return;
 
