@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -153,8 +153,8 @@ Dictionary::Ptr ServiceDbObject::GetStatusFields(void) const
 	fields->Set("percent_state_change", CompatUtility::GetCheckablePercentStateChange(service));
 
 	if (cr) {
-		fields->Set("latency", Convert::ToString(Service::CalculateLatency(cr)));
-		fields->Set("execution_time", Convert::ToString(Service::CalculateExecutionTime(cr)));
+		fields->Set("latency", Convert::ToString(cr->CalculateLatency()));
+		fields->Set("execution_time", Convert::ToString(cr->CalculateExecutionTime()));
 	}
 
 	fields->Set("scheduled_downtime_depth", service->GetDowntimeDepth());
@@ -188,6 +188,8 @@ void ServiceDbObject::OnConfigUpdate(void)
 		BOOST_FOREACH(const String& groupName, groups) {
 			ServiceGroup::Ptr group = ServiceGroup::GetByName(groupName);
 
+			std::vector<DbQuery> queries;
+
 			DbQuery query1;
 			query1.Table = DbType::GetByName("ServiceGroup")->GetTable() + "_members";
 			query1.Type = DbQueryDelete;
@@ -196,7 +198,7 @@ void ServiceDbObject::OnConfigUpdate(void)
 			query1.WhereCriteria->Set("instance_id", 0); /* DbConnection class fills in real ID */
 			query1.WhereCriteria->Set("servicegroup_id", DbValue::FromObjectInsertID(group));
 			query1.WhereCriteria->Set("service_object_id", service);
-			OnQuery(query1);
+			queries.push_back(query1);
 
 			DbQuery query2;
 			query2.Table = DbType::GetByName("ServiceGroup")->GetTable() + "_members";
@@ -206,7 +208,9 @@ void ServiceDbObject::OnConfigUpdate(void)
 			query2.Fields->Set("instance_id", 0); /* DbConnection class fills in real ID */
 			query2.Fields->Set("servicegroup_id", DbValue::FromObjectInsertID(group));
 			query2.Fields->Set("service_object_id", service);
-			OnQuery(query2);
+			queries.push_back(query2);
+
+			DbObject::OnMultipleQueries(queries);
 		}
 	}
 
