@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -39,6 +39,10 @@ boost::signals2::signal<void (const Checkable::Ptr&, const CheckResult::Ptr&, co
 boost::signals2::signal<void (const Checkable::Ptr&, const CheckResult::Ptr&, StateType, const MessageOrigin::Ptr&)> Checkable::OnStateChange;
 boost::signals2::signal<void (const Checkable::Ptr&, const CheckResult::Ptr&, std::set<Checkable::Ptr>, const MessageOrigin::Ptr&)> Checkable::OnReachabilityChanged;
 boost::signals2::signal<void (const Checkable::Ptr&, NotificationType, const CheckResult::Ptr&, const String&, const String&)> Checkable::OnNotificationsRequested;
+boost::signals2::signal<void (const Checkable::Ptr&)> Checkable::OnNextCheckUpdated;
+
+boost::mutex Checkable::m_StatsMutex;
+int Checkable::m_PendingChecks = 0;
 
 CheckCommand::Ptr Checkable::GetCheckCommand(void) const
 {
@@ -60,7 +64,7 @@ long Checkable::GetSchedulingOffset(void)
 	return m_SchedulingOffset;
 }
 
-void Checkable::UpdateNextCheck(void)
+void Checkable::UpdateNextCheck(const MessageOrigin::Ptr& origin)
 {
 	double interval;
 
@@ -75,7 +79,7 @@ void Checkable::UpdateNextCheck(void)
 	if (interval > 1)
 		adj = fmod(now * 100 + GetSchedulingOffset(), interval * 100) / 100.0;
 
-	SetNextCheck(now - adj + interval);
+	SetNextCheck(now - adj + interval, false, origin);
 }
 
 bool Checkable::HasBeenChecked(void) const
@@ -115,24 +119,14 @@ void Checkable::ProcessCheckResult(const CheckResult::Ptr& cr, const MessageOrig
 	if (cr->GetExecutionEnd() == 0)
 		cr->SetExecutionEnd(now);
 
-	if (!origin || origin->IsLocal()) {
-		Log(LogDebug, "Checkable")
-		    << "No origin or local origin for object '" << GetName()
-		    << "', setting " << IcingaApplication::GetInstance()->GetNodeName()
-		    << " as check_source.";
+	if (!origin || origin->IsLocal())
 		cr->SetCheckSource(IcingaApplication::GetInstance()->GetNodeName());
-	}
 
 	Endpoint::Ptr command_endpoint = GetCommandEndpoint();
 
 	/* override check source if command_endpoint was defined */
-	if (command_endpoint && !GetExtension("agent_check")) {
-		Log(LogDebug, "Checkable")
-		    << "command_endpoint found for object '" << GetName()
-		    << "', setting " << command_endpoint->GetName()
-		    << " as check_source.";
+	if (command_endpoint && !GetExtension("agent_check"))
 		cr->SetCheckSource(command_endpoint->GetName());
-	}
 
 	/* agent checks go through the api */
 	if (command_endpoint && GetExtension("agent_check")) {
@@ -159,6 +153,7 @@ void Checkable::ProcessCheckResult(const CheckResult::Ptr& cr, const MessageOrig
 	long old_attempt = GetCheckAttempt();
 	bool recovery = false;
 
+	/* Ignore check results older than the current one. */
 	if (old_cr && cr->GetExecutionStart() < old_cr->GetExecutionStart())
 		return;
 
@@ -168,19 +163,27 @@ void Checkable::ProcessCheckResult(const CheckResult::Ptr& cr, const MessageOrig
 	SetLastStateType(old_stateType);
 	SetLastReachable(reachable);
 
+	Host::Ptr host;
+	Service::Ptr service;
+	tie(host, service) = GetHostService(this);
+
+	CheckableType checkableType = CheckableHost;
+	if (service)
+		checkableType = CheckableService;
+
 	long attempt = 1;
 
 	std::set<Checkable::Ptr> children = GetChildren();
 
 	if (!old_cr) {
 		SetStateType(StateTypeHard);
-	} else if (cr->GetState() == ServiceOK) {
-		if (old_state == ServiceOK && old_stateType == StateTypeSoft) {
+	} else if (IsStateOK(cr->GetState())) {
+		if (IsStateOK(old_state) && old_stateType == StateTypeSoft) {
 			SetStateType(StateTypeHard); // SOFT OK -> HARD OK
 			recovery = true;
 		}
 
-		if (old_state != ServiceOK)
+		if (!IsStateOK(old_state))
 			recovery = true; // NOT OK -> SOFT/HARD OK
 
 		ResetNotificationNumbers();
@@ -192,17 +195,17 @@ void Checkable::ProcessCheckResult(const CheckResult::Ptr& cr, const MessageOrig
 	} else {
 		if (old_attempt >= GetMaxCheckAttempts()) {
 			SetStateType(StateTypeHard);
-		} else if (old_stateType == StateTypeSoft && old_state != ServiceOK) {
+		} else if (old_stateType == StateTypeSoft && !IsStateOK(old_state)) {
 			SetStateType(StateTypeSoft);
-			attempt = old_attempt + 1; //NOT-OK -> NOT-OK counter
-		} else if (old_state == ServiceOK) {
+			attempt = old_attempt + 1; // NOT-OK -> NOT-OK counter
+		} else if (IsStateOK(old_state)) {
 			SetStateType(StateTypeSoft);
-			attempt = 1; //OK -> NOT-OK transition, reset the counter
+			attempt = 1; // OK -> NOT-OK transition, reset the counter
 		} else {
 			attempt = old_attempt;
 		}
 
-		if (cr->GetState() != ServiceOK) {
+		if (!IsStateOK(cr->GetState())) {
 			SaveLastState(cr->GetState(), Utility::GetTime());
 		}
 
@@ -219,13 +222,20 @@ void Checkable::ProcessCheckResult(const CheckResult::Ptr& cr, const MessageOrig
 	ServiceState new_state = cr->GetState();
 	SetStateRaw(new_state);
 
-	bool stateChange = (old_state != new_state);
+	bool stateChange;
+
+	/* Exception on state change calculation for hosts. */
+	if (checkableType == CheckableService)
+		stateChange = (old_state != new_state);
+	else
+		stateChange = (Host::CalculateState(old_state) != Host::CalculateState(new_state));
+
 	if (stateChange) {
 		SetLastStateChange(now);
 
 		/* remove acknowledgements */
 		if (GetAcknowledgement() == AcknowledgementNormal ||
-		    (GetAcknowledgement() == AcknowledgementSticky && new_state == ServiceOK)) {
+		    (GetAcknowledgement() == AcknowledgementSticky && IsStateOK(new_state))) {
 			ClearAcknowledgement();
 		}
 
@@ -256,30 +266,32 @@ void Checkable::ProcessCheckResult(const CheckResult::Ptr& cr, const MessageOrig
 		SetLastHardStateChange(now);
 	}
 
-	if (new_state != ServiceOK)
+	if (!IsStateOK(new_state))
 		TriggerDowntimes();
 
-	Host::Ptr host;
-	Service::Ptr service;
-	tie(host, service) = GetHostService(this);
-
-	CheckableType checkable_type = CheckableHost;
-	if (service)
-		checkable_type = CheckableService;
-
 	/* statistics for external tools */
-	Checkable::UpdateStatistics(cr, checkable_type);
+	Checkable::UpdateStatistics(cr, checkableType);
 
 	bool in_downtime = IsInDowntime();
-	bool send_notification = hardChange && notification_reachable && !in_downtime && !IsAcknowledged();
+
+	bool send_notification = false;
+
+	if (notification_reachable && !in_downtime && !IsAcknowledged()) {
+		/* Send notifications whether when a hard state change occured. */
+		if (hardChange)
+			send_notification = true;
+		/* Or if the checkable is volatile and in a HARD state. */
+		else if (is_volatile && GetStateType() == StateTypeHard)
+			send_notification = true;
+	}
 
 	if (!old_cr)
 		send_notification = false; /* Don't send notifications for the initial state change */
 
-	if (old_state == ServiceOK && old_stateType == StateTypeSoft)
+	if (IsStateOK(old_state) && old_stateType == StateTypeSoft)
 		send_notification = false; /* Don't send notifications for SOFT-OK -> HARD-OK. */
 
-	if (is_volatile && old_state == ServiceOK && new_state == ServiceOK)
+	if (is_volatile && IsStateOK(old_state) && IsStateOK(new_state))
 		send_notification = false; /* Don't send notifications for volatile OK -> OK changes. */
 
 	bool send_downtime_notification = (GetLastInDowntime() != in_downtime);
@@ -307,10 +319,25 @@ void Checkable::ProcessCheckResult(const CheckResult::Ptr& cr, const MessageOrig
 	bool was_flapping, is_flapping;
 
 	was_flapping = IsFlapping();
+
 	if (GetStateType() == StateTypeHard)
 		UpdateFlappingStatus(stateChange);
+
 	is_flapping = IsFlapping();
 
+	if (cr->GetActive()) {
+		/* If there was a OK -> NOT-OK state change for actively scheduled checks,
+		 * update the next check time using the retry_interval.
+		 * Important: Add the cluster message origin. */
+		if (GetStateType() == StateTypeSoft)
+			UpdateNextCheck(origin);
+	} else {
+		/* Reschedule the next check for passive check results. The side effect of
+		 * this is that for as long as we receive passive results for a service we
+		 * won't execute any active checks. */
+		SetNextCheck(Utility::GetTime() + GetCheckInterval(), false, origin);
+	}
+
 	olock.Unlock();
 
 //	Log(LogDebug, "Checkable")
@@ -344,28 +371,24 @@ void Checkable::ProcessCheckResult(const CheckResult::Ptr& cr, const MessageOrig
 	if (send_downtime_notification)
 		OnNotificationsRequested(this, in_downtime ? NotificationDowntimeStart : NotificationDowntimeEnd, cr, "", "");
 
-	if (!was_flapping && is_flapping) {
-		OnNotificationsRequested(this, NotificationFlappingStart, cr, "", "");
+	if (send_notification) {
+		if (!was_flapping && is_flapping) {
+			OnNotificationsRequested(this, NotificationFlappingStart, cr, "", "");
 
-		Log(LogNotice, "Checkable")
-		    << "Flapping: Checkable " << GetName() << " started flapping (" << GetFlappingThreshold() << "% < " << GetFlappingCurrent() << "%).";
+			Log(LogNotice, "Checkable")
+				<< "Flapping: Checkable " << GetName() << " started flapping (" << GetFlappingThreshold() << "% < " << GetFlappingCurrent() << "%).";
 
-		NotifyFlapping(origin);
-	} else if (was_flapping && !is_flapping) {
-		OnNotificationsRequested(this, NotificationFlappingEnd, cr, "", "");
+			NotifyFlapping(origin);
+		} else if (was_flapping && !is_flapping) {
+			OnNotificationsRequested(this, NotificationFlappingEnd, cr, "", "");
 
-		Log(LogNotice, "Checkable")
-		    << "Flapping: Checkable " << GetName() << " stopped flapping (" << GetFlappingThreshold() << "% >= " << GetFlappingCurrent() << "%).";
+			Log(LogNotice, "Checkable")
+				<< "Flapping: Checkable " << GetName() << " stopped flapping (" << GetFlappingThreshold() << "% >= " << GetFlappingCurrent() << "%).";
 
-		NotifyFlapping(origin);
-	} else if (send_notification)
-		OnNotificationsRequested(this, recovery ? NotificationRecovery : NotificationProblem, cr, "", "");
-}
-
-bool Checkable::IsCheckPending(void) const
-{
-	ObjectLock olock(this);
-	return m_CheckRunning;
+			NotifyFlapping(origin);
+		} else if (!was_flapping && !is_flapping)
+			OnNotificationsRequested(this, recovery ? NotificationRecovery : NotificationProblem, cr, "", "");
+	}
 }
 
 void Checkable::ExecuteRemoteCheck(const Dictionary::Ptr& resolvedMacros)
@@ -386,6 +409,10 @@ void Checkable::ExecuteCheck(void)
 {
 	CONTEXT("Executing check for object '" + GetName() + "'");
 
+	/* keep track of scheduling info in case the check type doesn't provide its own information */
+	double scheduled_start = GetNextCheck();
+	double before_check = Utility::GetTime();
+
 	UpdateNextCheck();
 
 	bool reachable = IsReachable();
@@ -404,10 +431,6 @@ void Checkable::ExecuteCheck(void)
 		SetLastReachable(reachable);
 	}
 
-	/* keep track of scheduling info in case the check type doesn't provide its own information */
-	double scheduled_start = GetNextCheck();
-	double before_check = Utility::GetTime();
-
 	CheckResult::Ptr cr = new CheckResult();
 
 	cr->SetScheduleStart(scheduled_start);
@@ -448,12 +471,24 @@ void Checkable::ExecuteCheck(void)
 			if (listener)
 				listener->SyncSendMessage(endpoint, message);
 
-		} else if (Application::GetInstance()->GetStartTime() < Utility::GetTime() - 30) {
+			/* Re-schedule the check so we don't run it again until after we've received
+			   a check result from the remote instance. The check will be re-scheduled
+			   using the proper check interval once we've received a check result. */
+			SetNextCheck(Utility::GetTime() + GetCheckCommand()->GetTimeout() + 30);
+		} else if (Application::GetInstance()->GetStartTime() < Utility::GetTime() - 300) {
 			/* fail to perform check on unconnected endpoint */
 			cr->SetState(ServiceUnknown);
 
-			cr->SetOutput("Remote Icinga instance '" + endpoint->GetName() +
-			    "' " + "is not connected to '" + Endpoint::GetLocalEndpoint()->GetName() + "'");
+			String output = "Remote Icinga instance '" + endpoint->GetName() + "' is not connected to ";
+
+			Endpoint::Ptr localEndpoint = Endpoint::GetLocalEndpoint();
+
+			if (localEndpoint)
+				output += "'" + localEndpoint->GetName() + "'";
+			else
+				output += "this instance";
+
+			cr->SetOutput(output);
 
 			ProcessCheckResult(cr);
 		}
@@ -484,23 +519,20 @@ void Checkable::UpdateStatistics(const CheckResult::Ptr& cr, CheckableType type)
 	}
 }
 
-double Checkable::CalculateExecutionTime(const CheckResult::Ptr& cr)
+void Checkable::IncreasePendingChecks(void)
 {
-	if (!cr)
-		return 0;
-
-	return cr->GetExecutionEnd() - cr->GetExecutionStart();
+	boost::mutex::scoped_lock lock(m_StatsMutex);
+	m_PendingChecks++;
 }
 
-double Checkable::CalculateLatency(const CheckResult::Ptr& cr)
+void Checkable::DecreasePendingChecks(void)
 {
-	if (!cr)
-		return 0;
+	boost::mutex::scoped_lock lock(m_StatsMutex);
+	m_PendingChecks--;
+}
 
-	double latency = (cr->GetScheduleEnd() - cr->GetScheduleStart()) - CalculateExecutionTime(cr);
-
-	if (latency < 0)
-		latency = 0;
-
-	return latency;
+int Checkable::GetPendingChecks(void)
+{
+	boost::mutex::scoped_lock lock(m_StatsMutex);
+	return m_PendingChecks;
 }
