@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -86,8 +86,11 @@ CheckableCheckStatistics CIB::CalculateHostCheckStats(void)
 
 		CheckResult::Ptr cr = host->GetLastCheckResult();
 
+		if (!cr)
+			continue;
+
 		/* latency */
-		double latency = Host::CalculateLatency(cr);
+		double latency = cr->CalculateLatency();
 
 		if (min_latency == -1 || latency < min_latency)
 			min_latency = latency;
@@ -99,7 +102,7 @@ CheckableCheckStatistics CIB::CalculateHostCheckStats(void)
 		count_latency++;
 
 		/* execution_time */
-		double execution_time = Host::CalculateExecutionTime(cr);
+		double execution_time = cr->CalculateExecutionTime();
 
 		if (min_execution_time == -1 || execution_time < min_execution_time)
 			min_execution_time = execution_time;
@@ -135,8 +138,11 @@ CheckableCheckStatistics CIB::CalculateServiceCheckStats(void)
 
 		CheckResult::Ptr cr = service->GetLastCheckResult();
 
+		if (!cr)
+			continue;
+
 		/* latency */
-		double latency = Service::CalculateLatency(cr);
+		double latency = cr->CalculateLatency();
 
 		if (min_latency == -1 || latency < min_latency)
 			min_latency = latency;
@@ -148,7 +154,7 @@ CheckableCheckStatistics CIB::CalculateServiceCheckStats(void)
 		count_latency++;
 
 		/* execution_time */
-		double execution_time = Service::CalculateExecutionTime(cr);
+		double execution_time = cr->CalculateExecutionTime();
 
 		if (min_execution_time == -1 || execution_time < min_execution_time)
 			min_execution_time = execution_time;
@@ -311,6 +317,7 @@ void CIB::StatsFunc(const Dictionary::Ptr& status, const Array::Ptr& perfdata) {
 
 	status->Set("num_hosts_up", hs.hosts_up);
 	status->Set("num_hosts_down", hs.hosts_down);
+	status->Set("num_hosts_pending", hs.hosts_pending);
 	status->Set("num_hosts_unreachable", hs.hosts_unreachable);
 	status->Set("num_hosts_flapping", hs.hosts_flapping);
 	status->Set("num_hosts_in_downtime", hs.hosts_in_downtime);
