@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -38,6 +38,15 @@ void Host::OnAllConfigLoaded(void)
 {
 	ObjectImpl<Host>::OnAllConfigLoaded();
 
+	String zoneName = GetZoneName();
+
+	if (!zoneName.IsEmpty()) {
+		Zone::Ptr zone = Zone::GetByName(zoneName);
+
+		if (zone && zone->IsGlobal())
+			BOOST_THROW_EXCEPTION(std::invalid_argument("Host '" + GetName() + "' cannot be put into global zone '" + zone->GetName() + "'."));
+	}
+
 	HostGroup::EvaluateObjectRules(this);
 
 	Array::Ptr groups = GetGroups();
@@ -173,6 +182,11 @@ HostState Host::GetLastHardState(void) const
 	return CalculateState(GetLastHardStateRaw());
 }
 
+bool Host::IsStateOK(ServiceState state)
+{
+	return Host::CalculateState(state) == HostUp;
+}
+
 void Host::SaveLastState(ServiceState state, double timestamp)
 {
 	if (state == ServiceOK || state == ServiceWarning)
@@ -275,10 +289,10 @@ bool Host::ResolveMacro(const String& macro, const CheckResult::Ptr&, Value *res
 
 	if (cr) {
 		if (macro == "latency") {
-			*result = Convert::ToString(Service::CalculateLatency(cr));
+			*result = Convert::ToString(cr->CalculateLatency());
 			return true;
 		} else if (macro == "execution_time") {
-			*result = Convert::ToString(Service::CalculateExecutionTime(cr));
+			*result = Convert::ToString(cr->CalculateExecutionTime());
 			return true;
 		} else if (macro == "output") {
 			*result = cr->GetOutput();
