@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -88,7 +88,12 @@ void PluginUtility::ExecuteCommand(const Command::Ptr& commandObj, const Checkab
 		return;
 
 	Process::Ptr process = new Process(Process::PrepareCommand(command), envMacros);
-	process->SetTimeout(commandObj->GetTimeout());
+
+	if (checkable->GetCheckTimeout().IsEmpty())
+		process->SetTimeout(commandObj->GetTimeout());
+	else
+		process->SetTimeout(checkable->GetCheckTimeout());
+
 	process->Run(boost::bind(callback, command, _1));
 }
 
