@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -63,6 +63,15 @@ void Service::OnAllConfigLoaded(void)
 {
 	ObjectImpl<Service>::OnAllConfigLoaded();
 
+	String zoneName = GetZoneName();
+
+	if (!zoneName.IsEmpty()) {
+		Zone::Ptr zone = Zone::GetByName(zoneName);
+
+		if (zone && zone->IsGlobal())
+			BOOST_THROW_EXCEPTION(std::invalid_argument("Service '" + GetName() + "' cannot be put into global zone '" + zone->GetName() + "'."));
+	}
+
 	m_Host = Host::GetByName(GetHostName());
 
 	if (m_Host)
@@ -117,6 +126,11 @@ Host::Ptr Service::GetHost(void) const
 	return m_Host;
 }
 
+bool Service::IsStateOK(ServiceState state)
+{
+	return state == ServiceOK;
+}
+
 void Service::SaveLastState(ServiceState state, double timestamp)
 {
 	if (state == ServiceOK)
@@ -205,10 +219,10 @@ bool Service::ResolveMacro(const String& macro, const CheckResult::Ptr& cr, Valu
 
 	if (cr) {
 		if (macro == "latency") {
-			*result = Convert::ToString(Service::CalculateLatency(cr));
+			*result = Convert::ToString(cr->CalculateLatency());
 			return true;
 		} else if (macro == "execution_time") {
-			*result = Convert::ToString(Service::CalculateExecutionTime(cr));
+			*result = Convert::ToString(cr->CalculateExecutionTime());
 			return true;
 		} else if (macro == "output") {
 			*result = cr->GetOutput();
