@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -34,35 +34,48 @@ static bool ObjectNameLessComparer(const ConfigObject::Ptr& a, const ConfigObjec
 	return a->GetName() < b->GetName();
 }
 
-static void AuthorityTimerHandler(void)
+void ApiListener::UpdateObjectAuthority(void)
 {
-	ApiListener::Ptr listener = ApiListener::GetInstance();
-
-	if (!listener || !listener->IsActive())
-		return;
-
 	Zone::Ptr my_zone = Zone::GetLocalZone();
-	if (!my_zone)
-		return;
-
-	Endpoint::Ptr my_endpoint = Endpoint::GetLocalEndpoint();
 
 	std::vector<Endpoint::Ptr> endpoints;
-	BOOST_FOREACH(const Endpoint::Ptr& endpoint, my_zone->GetEndpoints()) {
-		if (!endpoint->GetConnected() && endpoint != my_endpoint)
-			continue;
+	Endpoint::Ptr my_endpoint;
 
-		endpoints.push_back(endpoint);
+	if (my_zone) {
+		my_endpoint = Endpoint::GetLocalEndpoint();
+
+		int num_total = 0;
+
+		BOOST_FOREACH(const Endpoint::Ptr& endpoint, my_zone->GetEndpoints()) {
+			num_total++;
+
+			if (endpoint != my_endpoint && !endpoint->GetConnected())
+				continue;
+
+			endpoints.push_back(endpoint);
+		}
+
+		double mainTime = Application::GetMainTime();
+
+		if (num_total > 1 && endpoints.size() <= 1 && (mainTime == 0 || Utility::GetTime() - mainTime < 60))
+			return;
+
+		std::sort(endpoints.begin(), endpoints.end(), ObjectNameLessComparer);
 	}
 
-	std::sort(endpoints.begin(), endpoints.end(), ObjectNameLessComparer);
-
 	BOOST_FOREACH(const ConfigType::Ptr& type, ConfigType::GetTypes()) {
 		BOOST_FOREACH(const ConfigObject::Ptr& object, type->GetObjects()) {
-			Endpoint::Ptr endpoint = endpoints[Utility::SDBM(object->GetName()) % endpoints.size()];
+			if (object->GetHAMode() != HARunOnce)
+				continue;
 
-			if (object->GetHAMode() == HARunOnce)
-				object->SetAuthority(endpoint == my_endpoint);
+			bool authority;
+
+			if (!my_zone)
+				authority = true;
+			else
+				authority = endpoints[Utility::SDBM(object->GetName()) % endpoints.size()] == my_endpoint;
+
+			object->SetAuthority(authority);
 		}
 	}
 }
@@ -70,7 +83,7 @@ static void AuthorityTimerHandler(void)
 static void StaticInitialize(void)
 {
 	l_AuthorityTimer = new Timer();
-	l_AuthorityTimer->OnTimerExpired.connect(boost::bind(&AuthorityTimerHandler));
+	l_AuthorityTimer->OnTimerExpired.connect(boost::bind(&ApiListener::UpdateObjectAuthority));
 	l_AuthorityTimer->SetInterval(30);
 	l_AuthorityTimer->Start();
 }
