@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -123,7 +123,10 @@ bool FilterUtility::EvaluateFilter(ScriptFrame& frame, Expression *filter,
 
 		Object::Ptr joinedObj = target->NavigateField(fid);
 
-		vars->Set(field.Name, joinedObj);
+		if (field.NavigationName)
+			vars->Set(field.NavigationName, joinedObj);
+		else
+			vars->Set(field.Name, joinedObj);
 	}
 
 	return Convert::ToBool(filter->Evaluate(frame));
