@@ -1,6 +1,6 @@
 /******************************************************************************
  * Icinga 2                                                                   *
- * Copyright (C) 2012-2015 Icinga Development Team (http://www.icinga.org)    *
+ * Copyright (C) 2012-2016 Icinga Development Team (https://www.icinga.org/)  *
  *                                                                            *
  * This program is free software; you can redistribute it and/or              *
  * modify it under the terms of the GNU General Public License                *
@@ -158,6 +158,33 @@ static bool FieldTypeCmp(const Field& a, const Field& b)
 	return a.Type.GetRealType() < b.Type.GetRealType();
 }
 
+static std::string FieldTypeToIcingaName(const Field& field, bool inner)
+{
+	std::string ftype = field.Type.TypeName;
+
+	if (!inner && field.Type.ArrayRank > 0)
+		return "Array";
+
+	if (field.Type.IsName)
+		return "String";
+
+	if (field.Attributes & FAEnum)
+		return "Number";
+
+	if (ftype == "bool" || ftype == "int" || ftype == "double")
+		return "Number";
+
+	if (ftype == "int" || ftype == "double")
+		return "Number";
+	else if (ftype == "bool")
+		return "Boolean";
+
+	if (ftype.find("::Ptr") != std::string::npos)
+		return ftype.substr(0, ftype.size() - strlen("::Ptr"));
+
+	return ftype;
+}
+
 void ClassCompiler::OptimizeStructLayout(std::vector<Field>& fields)
 {
 	std::sort(fields.begin(), fields.end(), FieldTypeCmp);
@@ -342,21 +369,7 @@ void ClassCompiler::HandleClass(const Klass& klass, const ClassDebugInfo&)
 
 		size_t num = 0;
 		for (it = klass.Fields.begin(); it != klass.Fields.end(); it++) {
-			std::string ftype = it->Type.GetRealType();
-
-			if (ftype == "bool" || ftype == "int" || ftype == "double")
-				ftype = "Number";
-
-			if (ftype == "int" || ftype == "double")
-				ftype = "Number";
-			else if (ftype == "bool")
-				ftype = "Boolean";
-
-			if (ftype.find("::Ptr") != std::string::npos)
-				ftype = ftype.substr(0, ftype.size() - strlen("::Ptr"));
-
-			if (it->Attributes & FAEnum)
-				ftype = "Number";
+			std::string ftype = FieldTypeToIcingaName(*it, false);
 
 			std::string nameref;
 
@@ -487,33 +500,39 @@ void ClassCompiler::HandleClass(const Klass& klass, const ClassDebugInfo&)
 
 		const Field& field = *it;
 
-		if ((field.Attributes & (FARequired)) || field.Type.IsName) {
-			if (field.Attributes & FARequired) {
-				if (field.Type.GetRealType().find("::Ptr") != std::string::npos)
-					m_Impl << "\t" << "if (!value)" << std::endl;
-				else
-					m_Impl << "\t" << "if (value.IsEmpty())" << std::endl;
+		if (field.Attributes & FARequired) {
+			if (field.Type.GetRealType().find("::Ptr") != std::string::npos)
+				m_Impl << "\t" << "if (!value)" << std::endl;
+			else
+				m_Impl << "\t" << "if (value.IsEmpty())" << std::endl;
 
-				m_Impl << "\t\t" << "BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), boost::assign::list_of(\"" << field.Name << "\"), \"Attribute must not be empty.\"));" << std::endl << std::endl;
-			}
+			m_Impl << "\t\t" << "BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), boost::assign::list_of(\"" << field.Name << "\"), \"Attribute must not be empty.\"));" << std::endl << std::endl;
+		}
 
-			if (field.Type.IsName) {
-				if (field.Type.ArrayRank > 0) {
-					m_Impl << "\t" << "if (value) {" << std::endl
-					       << "\t\t" << "ObjectLock olock(value);" << std::endl
-					       << "\t\t" << "BOOST_FOREACH(const String& ref, value) {" << std::endl;
-				} else
-					m_Impl << "\t" << "String ref = value;" << std::endl;
+		if (field.Type.ArrayRank > 0) {
+			m_Impl << "\t" << "if (value) {" << std::endl
+			       << "\t\t" << "ObjectLock olock(value);" << std::endl
+			       << "\t\t" << "BOOST_FOREACH(const Value& avalue, value) {" << std::endl;
+		} else
+			m_Impl << "\t" << "Value avalue = value;" << std::endl;
 
-				m_Impl << "\t" << "if (!ref.IsEmpty() && !utils.ValidateName(\"" << field.Type.TypeName << "\", ref))" << std::endl
-				       << "\t\t" << "BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), boost::assign::list_of(\"" << field.Name << "\"), \"Object '\" + ref + \"' of type '" << field.Type.TypeName
-				       << "' does not exist.\"));" << std::endl;
+		std::string ftype = FieldTypeToIcingaName(field, true);
 
-				if (field.Type.ArrayRank > 0) {
-					m_Impl << "\t\t" << "}" << std::endl
-					       << "\t" << "}" << std::endl;
-				}
-			}
+		if (field.Type.IsName) {
+			m_Impl << "\t" << "if (!avalue.IsEmpty() && !utils.ValidateName(\"" << field.Type.TypeName << "\", avalue))" << std::endl
+			       << "\t\t" << "BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), boost::assign::list_of(\"" << field.Name << "\"), \"Object '\" + avalue + \"' of type '" << field.Type.TypeName
+			       << "' does not exist.\"));" << std::endl;
+		} else if (field.Type.ArrayRank > 0 && (ftype == "Number" || ftype == "Boolean")) {
+			m_Impl << "\t" << "try {" << std::endl
+			       << "\t\t" << "Convert::ToDouble(avalue);" << std::endl
+			       << "\t" << "} catch (const std::invalid_argument&) {" << std::endl
+			       << "\t\t" << "BOOST_THROW_EXCEPTION(ValidationError(dynamic_cast<ConfigObject *>(this), boost::assign::list_of(\"" << field.Name << "\"), \"Array element '\" + avalue + \"' of type '\" + avalue.GetReflectionType()->GetName() + \"' is not valid here; expected type '" << ftype << "'.\"));" << std::endl
+			       << "\t" << "}" << std::endl;
+		}
+
+		if (field.Type.ArrayRank > 0) {
+			m_Impl << "\t\t" << "}" << std::endl
+			       << "\t" << "}" << std::endl;
 		}
 
 		m_Impl << "}" << std::endl << std::endl;
@@ -882,7 +901,7 @@ void ClassCompiler::HandleClass(const Klass& klass, const ClassDebugInfo&)
 			       << "\t" << klass.Parent << "::Start(runtimeCreated);" << std::endl << std::endl;
 
 			for (it = klass.Fields.begin(); it != klass.Fields.end(); it++) {
-				if (!(it->Type.IsName))
+				if (!it->Type.IsName && it->TrackAccessor.empty())
 					continue;
 
 				m_Impl << "\t" << "Track" << it->GetFriendlyName() << "(Empty, Get" << it->GetFriendlyName() << "());" << std::endl;
@@ -894,7 +913,7 @@ void ClassCompiler::HandleClass(const Klass& klass, const ClassDebugInfo&)
 			       << "\t" << klass.Parent << "::Stop(runtimeRemoved);" << std::endl << std::endl;
 
 			for (it = klass.Fields.begin(); it != klass.Fields.end(); it++) {
-				if (!(it->Type.IsName))
+				if (!it->Type.IsName && it->TrackAccessor.empty())
 					continue;
 
 				m_Impl << "\t" << "Track" << it->GetFriendlyName() << "(Get" << it->GetFriendlyName() << "(), Empty);" << std::endl;
@@ -916,11 +935,11 @@ void ClassCompiler::HandleClass(const Klass& klass, const ClassDebugInfo&)
 				 << "\t" << "virtual void Notify" << it->GetFriendlyName() << "(const Value& cookie = Empty);" << std::endl;
 
 			m_Impl << "void ObjectImpl<" << klass.Name << ">::Notify" << it->GetFriendlyName() << "(const Value& cookie)" << std::endl
-			       << "{" << std::endl
-			       << "\t" << "ConfigObject *dobj = dynamic_cast<ConfigObject *>(this);" << std::endl;
+			       << "{" << std::endl;
 
 			if (it->Name != "active") {
-				m_Impl << "\t" << "if (!dobj || dobj->IsActive())" << std::endl
+				m_Impl << "\t" << "ConfigObject *dobj = dynamic_cast<ConfigObject *>(this);" << std::endl
+				       << "\t" << "if (!dobj || dobj->IsActive())" << std::endl
 				       << "\t";
 			}
 
